##############################################
########### IMPORTING PACKAGES  ##############
##############################################

library(tseries)
# library(TSclust)
library(ggdendro) # dendrograms
library(gplots) # heatmap
library(cluster)
library(factoextra)

library(dtw)
library(proxy)
library(dtwclust)
library(NbClust)
library(clValid)
library(BBmisc)
library(kableExtra)

library(tidyr)
library(gridExtra)
# library(cowplot)

###########################################################
# Define working directory
###########################################################

current_file_dir <- dirname(rstudioapi::getActiveDocumentContext()$path)
setwd(current_file_dir)

# set the datasets folder as the working directory
setwd('../datasets')


##################################################
############### LOADING DATASETS #################
##################################################

seed <- 123
set.seed(seed)

time_series_df <- repos_month_date_intervals %>% filter(!is.na(medianRunDurationInMinutes) &
                                                          generalProjectCategory == 'No ML'
                                                        # generalProjectCategory == 'ML'
) %>% select(proj,
             loc_size,
             intervalOrder,
             medianRunDurationInMinutes)

# creates a list of medianRunDurationInMinutes values for each project
time_series.list <- lapply(split(time_series_df$medianRunDurationInMinutes, time_series_df$proj), as.numeric)

##################################################
# Data Preprocessing (Interpolating and Scaling) #
##################################################

# Linear Interpolation to set the same amount of elements to the time series of all projects
max_length <- max(lengths(time_series.list))
time_series.interpolated <- reinterpolate(time_series.list, new.length = max_length)
time_series.matrix.interpolated <- t(sapply(time_series.interpolated, unlist))

# This is a pre-modelling step. In this step, the data must be scaled or standardised 
# so that different attributes can be comparable. Standardised data has mean zero and 
# standard deviation one. we do this using scale() function
time_series.matrix.scaled <- t(scale(t(time_series.matrix.interpolated)))

# These plots is to verify whether the plot of the scaled matrix is similar to the one
# of the interpolated matrix, but within different scales
# plot(time_series.matrix.interpolated[1,])
# plot(time_series.matrix.scaled[1,])

##################################################
#      GETTING THE OPTMAL NUMBER OF CLUSTERS     #
#               GAP statistic                    #
##################################################

kmax <-  nrow(time_series.matrix.scaled) - 1

set.seed(123); system.time(
  gap_stat <- clusGap(time_series.matrix.scaled, FUN = kmeans,
                      nstart = 25, iter.max = 50,
                      K.max = kmax, B = 500)
)

# getting the optmal number of clusters based on gap statistic
best_k_clusters <- maxSE(f = gap_stat$Tab[, "gap"], SE.f = gap_stat$Tab[, "SE.sim"])
ggp <- fviz_gap_stat(gap_stat)
plot(ggp)

# ML with seed 123 gives clusgap = 10
# NoML with seed 123 gives clusgap = 7

##################################################
#     CREATING CLUSTERS BASED IN DTW DISTANCE    #
##################################################

# The number of clusters used in the clustering algorithm is based on gap statistic
# best_k_clusters <- 10 # based on gap results, the best k clusters for ML projects
best_k_clusters <- 7 # based on gap results, the best k clusters for No ML projects

# This code uses partititional algorithm to clustering the time series and shape as centroid
clusters <- tsclust(time_series.matrix.scaled, 
                    type="partitional",
                    seed=123, k=best_k_clusters,
                    centroid = "shape",
                    distance="dtw_basic")

# Ploting the genereted clusters (series + centroids)
# Note: It's possible to plot series or centroids separately by using type = "s" or type = "c"
plot(clusters, type = "sc")

##################################################
#        CUSTOMIZING THE CLESTERING PLOTS        #
##################################################

# This code customizes the clusters' plots by including the centroid in the black color.
# Also, we fit and plot a regression model based on the centroid of each cluster to help us 
# manually classify the clustes's trends in "increasing", "decreasing" or "maintaining".

# Getting and Converting the centroid matrix to a long format
time_series.centroids.matrix <- sapply(clusters@centroids, unlist)
# data_long_centroids <- gather(time_series.centroids.df, key = "Column", value = "Value")

# This list stores the customized plots of each cluster generated by the clustering algorithm
clusters_plots <- list()

# This list stores the classification of the centroid trends for each cluster
clusters.trends.classification <- c()

# Iterate through each cluster
index <- 1
for (cluster_id in sort(unique(clusters@cluster))) {
  
  # Extract cluster members
  members_index <- which(cluster_id == clusters@cluster)
  
  # Filtering the time series grouped in the current cluster (cluster_id)
  if(length(members_index) == 1){
    # When the cluster has only one time series we have to ensure that the time series
    # values will be put into a data.frame instead of a vector
    colname <- rownames(time_series.matrix.scaled)[members_index]
    cluster_df <- data.frame(time_series.matrix.scaled[members_index,])
    colnames(cluster_df) <- colname
  }else{
    cluster_df <- data.frame(t(time_series.matrix.scaled[members_index,]))  
  }
  
  cluster_df.list <- as.list(cluster_df)
  
  long_df <- enframe(cluster_df.list, name = "Variable", value = "Values") %>%
    unnest(Values)
  
  # Create a new column with the sequence that restarts when "Variable" changes
  long_df <- long_df %>%
    group_by(Variable) %>%
    mutate(Sequence = row_number())
  
  # Calculate centroid
  centroid <- time_series.centroids.matrix[,c(cluster_id)]
  
  # Create a data frame for the centroid
  centroid_df <- data.frame(
    Sequence = 1:length(centroid),
    Centroid = centroid
  )
  
  # Fit a linear model to the centroid values
  lm_result <- lm(centroid_df$Centroid ~ centroid_df$Sequence)
  
  # Extract intercept and slope coefficients
  intercept <- coef(lm_result)[1]
  slope <- coef(lm_result)[2]
  
  # This classification is only to help us to classify the centroid trend
  # However, we double check the centroid plot manually to verify if its
  # necessary to change the trend classification 
  cluster_trend <- NA
  if(slope > 0.025){
    cluster_trend <- "Increasing"
  }else if(slope < -0.025){
    cluster_trend <- "Decreasing"
  }else{
    cluster_trend <- "Maintaining"
  }
  
  clusters.trends.classification <- c(clusters.trends.classification, cluster_trend)
  
  print(paste("Cluster", cluster_id, "-", cluster_trend, "- Slope:",slope))
  
  # -----------------------------------------------------
  # Generating the cluster plot object (p) 
  # to be stored in the clusters_plots list
  # -----------------------------------------------------
  plot_title <- paste("Cluster", cluster_id)
  
  # Convert the cluster time series matrix to a data frame
  cluster_df <- as.data.frame(t(time_series.matrix.scaled[members_index,]))
  # cluster_df$time_step <- 1:nrow(cluster_df)
  
  # Plot the data as time series with the added Sequence column
  p <- ggplot() +
    geom_line(data = long_df, aes(x = Sequence, y = Values, group = Variable, color = Variable)) +
    geom_line(data = centroid_df, aes(x = Sequence, y = Centroid), 
              color = "black", linetype = "dashed", linewidth = 1.5) +
    labs(x = "", y = "") +
    theme_minimal() +
    theme(legend.position = "none",
          axis.title.x = element_blank(), 
          axis.text.x = element_blank(),
          plot.title = element_text(hjust = 0.5,
                                    face="bold"),
          plot.subtitle=element_text(hjust=0.5)) +
    labs(title=plot_title,
         subtitle=cluster_trend) +
    geom_abline(intercept = intercept, slope = slope, color = "red", linewidth = 1.0)
  # labs(title=plot_title) +
  # scale_y_continuous(limits = c(-2, 5)) +
  # labs(title=plot_title,
  #      subtitle=cluster_trend)
  
  
  clusters_plots[[index]] <- p
  index <- index + 1
}

# ---------------------------------------------------------------
# Ploting the customized clusters' plots from clusters_plots list
# ---------------------------------------------------------------

# Arrange plots using gridExtra
arranged_plots <- grid.arrange(
  grobs = clusters_plots,
  ncol = 3
)

# Display the arranged plots
# print(arranged_plots)

# ---------------------------------------------------------
# cluster trend manual (qualitative) classification
# ---------------------------------------------------------

# if the manual inspection on the clusters' trends identify classification mistake
# manually correct the trend classification for the cluster and create the plots again
# E.g., >> clusters.trends.classification[4] <- "Maintaining"

# ---------------------------------------------------------
# cluster trend manual (qualitative) classification
# ---------------------------------------------------------

proj_cluster <- data.frame(fullName = names(clusters@datalist), 
                           cluster = clusters@cluster)

proj_cluster$trend <- NA
for(cluster_id in 1:length(clusters.trends.classification)){
  # print(proj_cluster$cluster)
  proj_cluster[proj_cluster$cluster == cluster_id, c("trend")] <- clusters.trends.classification[cluster_id]
}


sqldf("select trend, 
              COUNT(*) as count,
              COUNT(*) * 100.0 / SUM(COUNT(*)) OVER () AS percentage
      from proj_cluster
      group by trend")

# ML
# trend count percentage
# 1  Decreasing    19   20.43011
# 2  Increasing    54   58.06452
# 3 Maintaining    20   21.50538

# No ML
# trend count percentage
# 1  Decreasing    20   21.73913
# 2  Increasing    48   52.17391
# 3 Maintaining    24   26.08696


proj_size <- sqldf("select proj, loc_size
                    from time_series_df
                    group by proj, loc_size")


result <- inner_join(proj_cluster, select(repos_data, 
                                          fullName, 
                                          loc_size, 
                                          generalProjectCategory), 
                     by = "fullName")

write.csv(result, file = "RQ2-no_ml_projects_clustering_trends.csv", row.names = FALSE, na = c('NULL'))

sqldf("select trend, loc_size, count(*) as projects
      from result
      group by trend, loc_size")


query <- "
  SELECT 
    r.trend, r.loc_size, 
    COUNT(*) as projects,
    (COUNT(*) * 100.0) / t.total_count as percentage
  FROM 
    result AS r
  JOIN (
    SELECT loc_size, COUNT(*) as total_count
    FROM result
    GROUP BY loc_size
  ) AS t
  ON r.loc_size = t.loc_size
  GROUP BY 
    r.trend, 
    r.loc_size
"

# Execute the query using sqldf
df <- sqldf(query)
View(df)

# ML
# > df
#         trend loc_size projects percentage
# 1  Decreasing    large       10   24.39024
# 2  Decreasing   medium        7   15.90909
# 3  Decreasing    small        2   25.00000
# 4  Increasing    large       21   51.21951
# 5  Increasing   medium       27   61.36364
# 6  Increasing    small        6   75.00000
# 7 Maintaining    large       10   24.39024
# 8 Maintaining   medium       10   22.72727


# No ML
# > df
#         trend loc_size projects percentage 
# 1  Decreasing    large        7   17.50000
# 2  Decreasing   medium        8   21.05263
# 3  Decreasing    small        5   35.71429
# 4  Increasing    large       26   65.00000
# 5  Increasing   medium       17   44.73684
# 6  Increasing    small        5   35.71429
# 7 Maintaining    large        7   17.50000
# 8 Maintaining   medium       13   34.21053
# 9 Maintaining    small        4   28.57143


# Sample data for ML projects
df_ml <- data.frame(
  trend = c("Decreasing", "Decreasing", "Decreasing", "Increasing", "Increasing", "Increasing", "Maintaining", "Maintaining"),
  loc_size = c("large", "medium", "small", "large", "medium", "small", "large", "medium"),
  projects = c(10, 7, 2, 21, 27, 6, 10, 10),
  percentage = c(24.39024, 15.90909, 25.00000, 51.21951, 61.36364, 75.00000, 24.39024, 22.72727),
  category = "ML"
)

# Sample data for non-ML projects
df_non_ml <- data.frame(
  trend = c("Decreasing", "Decreasing", "Decreasing", "Increasing", "Increasing", "Increasing", "Maintaining", "Maintaining", "Maintaining"),
  loc_size = c("large", "medium", "small", "large", "medium", "small", "large", "medium", "small"),
  projects = c(7, 8, 5, 26, 17, 5, 7, 13, 4),
  percentage = c(17.50000, 21.05263, 35.71429, 65.00000, 44.73684, 35.71429, 17.50000, 34.21053, 28.57143),
  category = "No ML"
)

# Combine the data frames
df <- rbind(df_ml, df_non_ml)

# Define the order of the trends
trend_order <- c("Increasing", "Decreasing", "Maintaining")

# Reorder the factor levels
df$trend <- reorder(df$trend,
                    as.numeric(factor(df$trend, 
                                      levels = trend_order)))


# Define the order of the trends
loc_size_order <- c("small", "medium", "large")

# Reorder the factor levels
df$loc_size <- reorder(df$loc_size,
                       as.numeric(factor(df$loc_size, 
                                         levels = loc_size_order)))

# Define custom colors for the fill scale
custom_colors <- c("#F8766D", "#00BFC4", "gray60") # #7CAE00

# Calculate the position of labels for percentage values
df$label_pos <- cumsum(df$percentage) - df$percentage / 2

# Plotting with loc_size facets side by side
ggplot(df, aes(x = category, y = percentage, fill = trend)) +
  geom_bar(position = "fill", stat = "identity") +
  geom_text(aes(label = paste0(round(percentage, 1), "%")), 
            position = position_fill(vjust = 0.5), size = 4, color = "black") +
  facet_grid(. ~ loc_size, scales = "free_x", space = "free_x") +
  scale_y_continuous(labels = scales::percent_format(scale = 100)) +
  scale_fill_manual(values = custom_colors, name = "Build Duration Trend") +
  labs(x = NULL, y = "Percentage", title = NULL) +
  theme_minimal() +
  theme(axis.text.x = element_text(size = 12),
        strip.text = element_text(size = 12))  # Increase facet_grid text size
