library(datetime)
library(lubridate)
library(sqldf)
library(ggplot2)
library(rtern)
library(beanplot)
library(effsize)
library(tidyverse)
library(dplyr)
library(tidyverse)
library(ggpubr)
library(rstatix)
library(data.table)
library(grid)
library(gridExtra)
library(changepoint)

###########################################################
# Define work directory
###########################################################

current_file_dir <- dirname(rstudioapi::getActiveDocumentContext()$path)
setwd(current_file_dir)

# set the datasets folder as the working directory
setwd('../datasets')

###########################################################
# Import repository dataset
###########################################################

repos_data <- read.csv2("repositories-info.csv", sep = ",", dec=".", na.strings = c('NULL'))

# View(repos_data)

###########################################################
# Define the project size based on their lines of codes
###########################################################

repos_data['loc_size'] <- NA
repos_data[repos_data$lines_of_code <= 10000, 'loc_size'] <- 'small'
repos_data[repos_data$lines_of_code >  10000   & repos_data$lines_of_code <=  100000, 'loc_size'] <- 'medium'
repos_data[repos_data$lines_of_code >  100000, 'loc_size'] <- 'large'

# Define the order of the boxes
box_order <- c("ML", "No ML")

# Define the order of the boxes
loc_size_box_order <- c("small", "medium", "large")

# Reorder the factor levels
repos_data$generalProjectCategory <- reorder(repos_data$generalProjectCategory, 
                                             as.numeric(factor(repos_data$generalProjectCategory, 
                                                               levels = box_order)))

# Reorder the factor levels
repos_data$loc_size <- reorder(repos_data$loc_size, 
                               as.numeric(factor(repos_data$loc_size, 
                                                 levels = loc_size_box_order)))

# --------------------------------------------------------------------------
# Summarize repositories count by generalProjectCategory and loc_size
# --------------------------------------------------------------------------

# number of repositories per category (AI-ML, No AI-ML)
sqldf("select generalProjectCategory category, count(*) projects_count
            from repos_data repos
            group by generalProjectCategory
            ")

# number of repositories per category (AI-ML, No AI-ML) 
sqldf("select generalProjectCategory category, isOutOfMLDataset, count(*) projects_count
            from repos_data repos
            group by generalProjectCategory, isOutOfMLDataset
            ")

# number of repositories per category and size
sqldf("select generalProjectCategory AS category, loc_size, count(*) projects_count
      from repos_data repos
      group by generalProjectCategory, loc_size
      order by loc_size
      ")

# number of repositories per category and programming language 
sqldf("select generalProjectCategory, language, count(*) projects_count
      from repos_data repos
      group by generalProjectCategory, language
      ORDER BY generalProjectCategory, projects_count DESC
      ")

###########################################################
# Import commits dataset
###########################################################

# The commits dataset has some fields with double quotes within them.
# The read.CSV function doesn't deal with this specific format
# We have to replace the double quotes before reading the data

lines <- readLines("commits-info.csv")
lines <- gsub('(?<!,)"(?!,)', '', lines, perl = TRUE)

commits_data <- read.csv2(textConnection(lines), sep = ",", na.strings = c('NULL'))
commits_data <- commits_data[commits_data$fullName %in% unique(repos_data$fullName),]

commits_data$weekDay <- lubridate::wday(commits_data$authorDate, label = TRUE, abbr = FALSE)
commits_data$weekDayNumber <- lubridate::wday(commits_data$authorDate, week_start = 1)
commits_data$month <- lubridate::month(commits_data$authorDate, label = TRUE, abbr = FALSE)

# It is neccessary to change commiterDate class from POSIXlt to character
# because sqldf function does not support datatables with POSIXlt columns
commits_data$committerDate <- as.character(commits_data$committerDate)
commits_data$authorDate <- as.character(commits_data$authorDate)

###########################################################
## Import GHA run dataset
###########################################################

# import GHA run dataset with time to fix metric calculated
# if the time to fix is not calculated, then calculate

gha_runs_data_time_to_fix_file_name <- "gha-runs-info_time-to-fix-calculated.csv"

if(file.exists(gha_runs_data_time_to_fix_file_name)){
  gha_runs_data <- read.csv2(gha_runs_data_time_to_fix_file_name, sep = ",", dec=".", na.strings = c('NULL'))
}else{
  gha_runs_data <- read.csv2("gha-runs-info.csv", sep = ",", dec=".", na.strings = c('NULL'))
  
  # filter out workflow run duration higher than 35 days, the limit defined by GH
  # https://docs.github.com/en/actions/learn-github-actions/usage-limits-billing-and-administration
  
  gha_runs_data <- gha_runs_data[difftime(gha_runs_data$updatedAt,
                                          gha_runs_data$run_started_at, 
                                          units = "days") <= 35,]
  
  ## calculate the time to fix broken runs by repository
  calculateTimeToFixBrokenRunByRepos <- function(workflowRuns){
    
    workflowRuns <- workflowRuns[order(workflowRuns$workflow_id, workflowRuns$updatedAt),]
    
    workflowRuns$fixedBy <- NA
    workflowRuns$fixedAt <- NA
    workflowRuns$timeToFix <- NA
    
    if(nrow(workflowRuns) != 0){
      failed_run_date <- NA
      failed_run <- NA
      failed_run_index <- NA
      current_wokflow_id <- workflowRuns[1,]$workflow_id
      
      for(i in 1:nrow(workflowRuns)){
        run <- workflowRuns[i,]
        
        if(current_wokflow_id != run$workflow_id){
          failed_run_date <- NA
          failed_run <- NA
          failed_run_index <- NA
          current_wokflow_id <- run$workflow_id
        }
        
        if(is.na(failed_run_date) && run$conclusion %in% c("failure")){
          failed_run_date <- run$updatedAt
          failed_run_index <- i
          # print(paste("Failed Run date: ", failed_run_date))
        }
        if(!is.na(failed_run_date) && run$conclusion == "success"){
          passed_run_date <- run$updatedAt
          time_to_fix_run <- as.numeric(difftime(passed_run_date, failed_run_date, units = "hours")) / 24 # in days
          failed_run_date <- NA
          workflowRuns[failed_run_index,]$fixedBy <- run$ghId
          workflowRuns[failed_run_index,]$fixedAt <- run$updatedAt
          workflowRuns[failed_run_index,]$timeToFix <- time_to_fix_run
        }
      }
    }
    return(workflowRuns)
  }
  
  # calculate the time to fix broken runs for all repositories
  calculateTimeToFixBrokenRunAllRepos <- function(runs){
    runs_aux = data.frame()
    runs_colnames <- NA
    for(proj in unique(runs$fullName)){
      proj_runs <- runs[runs$fullName == proj,]
      proj_runs <- calculateTimeToFixBrokenRunByRepos(proj_runs)
      runs_aux <- rbind(runs_aux, proj_runs)
      runs_colnames <- names(runs_aux)
    }
    colnames(runs_aux) <- runs_colnames
    return(runs_aux)
  }
  
  ## adding the time to fix broken run for each studied project
  
  gha_runs_data <- calculateTimeToFixBrokenRunAllRepos(gha_runs_data)
  gha_runs_data$timeToFixInHours <- gha_runs_data$timeToFix * 24
  
  write.csv(gha_runs_data, file = "gha-runs-info_time-to-fix-calculated.csv", row.names = FALSE, sep = ",", dec = ".", na = c('NULL'))  
}

###########################################################
## Importing Coverage info - From coveralls and CodeCov
###########################################################

coverage_data <- read.csv2("coverage-info.csv", sep = ",", dec=".", na.strings = c('NULL'))

###########################################################
## Importing Pull Requests info
###########################################################

# Parse the cleaned JSON
# pulls_data_json <- jsonlite::fromJSON("pulls-info.json")
# 
# # Convert the JSON data to dataframe 
# pulls_data <- as.data.frame(pulls_data_json) 

pulls_data <- read.csv2("pulls-info.csv", sep = ",", dec=".", na.strings = c('NULL'))

pulls_data$isDraft <- as.logical(as.numeric(pulls_data$isDraft))
pulls_data$locked <- as.logical(as.numeric(pulls_data$locked))
pulls_data$mergeable <- as.logical(as.numeric(pulls_data$mergeable))
pulls_data$merged <- as.logical(as.numeric(pulls_data$merged))

pulls_data$mergeTimeInHours <- round(difftime(as.POSIXct(pulls_data$mergedAt), as.POSIXct(pulls_data$createdAt), units = "hours"), digits = 2)
pulls_data$mergeTimeInDays <- round(difftime(as.POSIXct(pulls_data$mergedAt), as.POSIXct(pulls_data$createdAt), units = "days"), digits = 3)


###########################################################
## Summaryzing projects general information
###########################################################

pulls_per_project <- summarise(
  group_by(pulls_data, fullName),
  pull_requests = length(fullName))

coverage_per_project <- summarise(
  group_by(coverage_data, fullName),
  coverage_builds = length(fullName))

commits_per_project <- summarise(
  group_by(commits_data, fullName),
  commits = length(fullName))

ghruns_per_project <- summarise(
  group_by(gha_runs_data, fullName),
  ci_builds = length(fullName))


# Add the repository general info (i.e., count the number of pulls per project) to the repository dataset
repos_data <- left_join(repos_data,
                        pulls_per_project %>% dplyr::select(fullName,
                                                            pull_requests),
                        by = "fullName")


repos_data <- left_join(repos_data,
                        coverage_per_project %>% dplyr::select(fullName,
                                                               coverage_builds),
                        by = "fullName")

repos_data <- left_join(repos_data,
                        commits_per_project %>% dplyr::select(fullName,
                                                              commits),
                        by = "fullName")

repos_data <- left_join(repos_data,
                        ghruns_per_project %>% dplyr::select(fullName,
                                                             ci_builds),
                        by = "fullName")

# Calculate the project age based on the creation_at attribute of the project and the date when the search was done in GitHub
date_of_github_search <- as.Date("2023-06-02")
repos_data$ageInYear <- as.numeric(round(difftime(as.POSIXct(date_of_github_search), as.POSIXct(repos_data$created_at), units = "days") / 365, 1))

summarise(group_by(repos_data, loc_size, generalProjectCategory),
          projects = length(loc_size),
          forks = median(forks, na.rm = TRUE),
          watchers = median(watchers, na.rm = TRUE),
          medianAgeYears = median(ageInYear),
          coverage_builds = sum(coverage_builds, na.rm = TRUE),
          pull_requests = sum(pull_requests),
          commits = sum(commits),
          ci_builds = sum(ci_builds),
          contributors = median(totalContributorsGitHubPage, na.rm = TRUE),
          releases = median(totalReleasesGitHubPage, na.rm = TRUE),
          sloc = median(lines_of_code)
)

summarise(group_by(repos_data, generalProjectCategory),
          medianAgeYears = median(ageInYear),
          medianForks = median(forks, na.rm = TRUE),
          medianWatchers = median(watchers, na.rm = TRUE),
          medianContributors = median(totalContributorsGitHubPage, na.rm = TRUE),
)

###########################################################
#
# Calculate monthly intervals by project
#
###########################################################

# --------------------------------------------------------------------------
# Declaration of functions used to measure information to the monthly intervals of the projects 
# --------------------------------------------------------------------------

## This function calculate the number of occurrences of weekend days in a date interval
countWeekendDaysInInterval <- function(start_date, end_date){
  myDates <-seq(from = start_date, to = end_date, by = "days")
  weekdays <- c("Saturday", "Sunday")
  return(length(which(lubridate::wday(myDates, label = TRUE, abbr = FALSE) %in% weekdays)))
}

## This function counts the different commiters that commited to a set of commits
countAuthorsInCommits <- function(commits_data){
  authorsCount <- 0
  if(nrow(commits_data) > 0){
    authors <- unique(commits_data$authorEmail)  
    authors <- authors[!authors == 'noreply@github.com']
    authorsCount <- length(authors)
  }
  return(authorsCount)
}

## This function gets the last code coverage information available in a set of coverages of a date interval
lastCoverageInIntervalMetric <- function(coverages){
  coverages <- coverages[!is.na(coverages$covered_percent),]
  sortedCoveragesInInterval <- coverages[order(coverages$created_at), ]
  if(nrow(sortedCoveragesInInterval) > 0){
    return(sortedCoveragesInInterval[nrow(sortedCoveragesInInterval),]$covered_percent)  
  }
  return(NA)
}

## This function calculates the Commit Activity Metric 
# Commit Activity Metric is a unit interval representing the rate of commits across days. 
# If commits were made every day in a period (1 month), the value would be 1. 
# If commits were made in half of days the value would be 0.5. 
# If there were no commits in a period the value would be 0.
calculateCommitActivityMetric <- function(commits, start_date, end_date, excludeWeekendDays=FALSE){
  dates <- seq(from = start_date, to = end_date, by = "days")
  days_in_interval = length(dates)
  if(excludeWeekendDays){
    weekendDaysInInterval <- countWeekendDaysInInterval(start_date, end_date)
    days_in_interval <- days_in_interval - weekendDaysInInterval
  }
  days_with_commits <- length(unique(as.Date(commits$committerDate)))
  if(days_in_interval > 0){
    return(days_with_commits / days_in_interval)  
  }
  return(NA)
}

## This function calculates the Build Health Metric 
# Build Health is a unit interval representing the rate of build failures across days. 
# If there were build failures every day, the value would be 0. 
# if there were no build failures, the value would be 1.
calculateRunsHealthMetric <- function(runsInInterval, start_date, end_date, excludeWeekendDays=FALSE){
  dates <- seq(from = start_date, to = end_date, by = "days")
  days_in_interval = length(dates)
  if(excludeWeekendDays){
    weekendDaysInInterval <- countWeekendDaysInInterval(start_date, end_date)
    days_in_interval <- days_in_interval - weekendDaysInInterval
  }
  days_with_failed_builds <- length(unique(as.Date(runsInInterval[runsInInterval$conclusion == "failure",]$createdAt)))
  if(days_in_interval > 0){
    return(1 - days_with_failed_builds / days_in_interval)  
  }
  return(NA)
}

## This function calculates the Build Health Metric 
# Build Health is a unit interval representing the rate of build failures across days that had builds. 
# If there were build failures every day that a build was triggered, the value would be 0. 
# if there were no build failures, the value would be 1.
calculateRunsHealthMetricAdapted <- function(runsInInterval, start_date, end_date){
  days_with_builds <- length(unique(as.Date(runsInInterval$createdAt)))
  days_with_failed_builds <- length(unique(as.Date(runsInInterval[runsInInterval$conclusion == "failure",]$createdAt)))
  if(days_with_builds > 0){
    return(1 - days_with_failed_builds / days_with_builds)  
  }
  return(NA)
}

## This function calculates the Runs Activity Metric 
# Runs Activity is a unit interval (i.e., a closed interval [0,1]) representing the rate of runs across days, 
# i.e, if runs were made every day in the period of analysis, the value would be 1. 
# If runs were made in half of the days, the value would be 0.5. 
# If there were no runs, the value would be 0.
calculateRunsActivityMetric <- function(runsInInterval, start_date, end_date, excludeWeekendDays=FALSE){
  dates <- seq(from = start_date, to = end_date, by = "days")
  days_in_interval = length(dates)
  if(excludeWeekendDays){
    weekendDaysInInterval <- countWeekendDaysInInterval(start_date, end_date)
    days_in_interval <- days_in_interval - weekendDaysInInterval
  }
  days_with_builds <- length(unique(as.Date(runsInInterval$createdAt)))
  if(days_in_interval > 0){
    return(days_with_builds / days_in_interval)  
  }
  return(NA)
}

# --------------------------------------------------------------------------
# calculating the monthly interval for the studied projects
# --------------------------------------------------------------------------

repos_month_date_intervals <- data.frame()

repos_data$monthlyIntervals <- NA
interval_lenght_in_days <- 30
for(proj in unique(repos_data$fullName)){
  
  # getting the project specific data (first and last GHA Run)
  projData <- repos_data[repos_data$fullName == proj,]
  loc_size <- projData$loc_size
  generalProjectCategory <- projData$generalProjectCategory
  
  # getting the project specific data (commits and workflowsRuns)
  projCommitsData <- commits_data[commits_data$fullName == proj,]
  projRunsData <- gha_runs_data[gha_runs_data$fullName == proj,]
  projCoverageData <- coverage_data[coverage_data$fullName == proj,]
  projPullsData <- pulls_data[pulls_data$fullName == proj,]
  # projWorkflowFileChangeData <- workflow_file_change_data[workflow_file_change_data$fullName == proj,]
  
  firstCIWorkflowRunAt <- as.Date(min(projRunsData$createdAt))
  lastCIWorkflowRunAt <- as.Date(max(projRunsData$createdAt))
  
  start_date <- firstCIWorkflowRunAt
  
  monthIntervals <- 0
  ## start
  while(start_date + days(interval_lenght_in_days - 1) < lastCIWorkflowRunAt){
    end_date <- start_date + days(interval_lenght_in_days - 1)
    print(sprintf("Proj: %s - Start date: %s - End date: %s", proj, start_date, end_date))
    
    commitsInInterval <- projCommitsData[as.Date(projCommitsData$committerDate) >= start_date &
                                           as.Date(projCommitsData$committerDate) <= end_date,]
    
    runsInInterval <- projRunsData[as.Date(projRunsData$createdAt) >= start_date &
                                     as.Date(projRunsData$createdAt) <= end_date,]
    
    successfullyRunsInInterval <- runsInInterval[runsInInterval$conclusion %in% c("success"),]
    
    successfullyRunsInIntervalCount <- nrow(successfullyRunsInInterval)
    
    failedRunsInIntervalCount <- nrow(runsInInterval[runsInInterval$conclusion %in% c("failure"),])
    
    successfulAndFailedRunsInInterval <- runsInInterval[runsInInterval$conclusion %in% c("success", "failure"),]
    
    otherRunsInIntervalCount <- nrow(runsInInterval[!(runsInInterval$conclusion %in% c("success", "failure")),])
    
    coverageInInterval <- projCoverageData[as.Date(projCoverageData$created_at) >= start_date &
                                             as.Date(projCoverageData$created_at) <= end_date,]
    
    fixed_runsInInterval <- runsInInterval[!is.na(runsInInterval$timeToFixInHours),]
    
    runsInIntervalCount <- nrow(runsInInterval)
    fixedRunsInInterval <- nrow(fixed_runsInInterval)
    coverageBuildsInInterval <- nrow(coverageInInterval)
    
    noNARunDurations <- runsInInterval[!is.na(runsInInterval$duration) & runsInInterval$duration >= 0,]$duration
    
    medianRunDurationInMinutes <- NA
    medianTimeToFixRunsInHours <- NA
    runsHealth <- NA
    
    runsActivity <- calculateRunsActivityMetric(runsInInterval, start_date, end_date)
    
    if(nrow(successfulAndFailedRunsInInterval) > 0){
      runsHealth <- calculateRunsHealthMetricAdapted(successfulAndFailedRunsInInterval, start_date, end_date)
    }
    
    if(successfullyRunsInIntervalCount > 0){
      medianRunDurationInMinutes <- median(successfullyRunsInInterval$duration) / 60
    }
    
    if(fixedRunsInInterval > 0){
      medianTimeToFixRunsInHours <- median(fixed_runsInInterval$timeToFixInHours)
    }
    
    commitsCount <- nrow(commitsInInterval)
    
    weekendDaysInInterval <- countWeekendDaysInInterval(start_date, end_date)
    
    authors <- countAuthorsInCommits(commitsInInterval)
    
    commitRatePerDay <- NA
    commitRateExcludingWeekendDays <- NA
    commitsPerAuthor <- NA
    commitActivity <- NA
    
    # if(commitsCount > 0){
    commitRatePerDay <- commitsCount / interval_lenght_in_days
    commitRatePerDayExcludingWeekendDays <- commitsCount / (interval_lenght_in_days - weekendDaysInInterval)
    commitsPerAuthor <- commitsCount / authors
    commitActivity <- calculateCommitActivityMetric(commitsInInterval, start_date, end_date)
    # }
    
    lastCoverageInInterval <- NA
    if(coverageBuildsInInterval > 0){
      lastCoverageInInterval <- lastCoverageInIntervalMetric(coverageInInterval)
    }
    
    submitedPRs_df <- projPullsData[as.Date(projPullsData$createdAt) >= start_date &
                                    as.Date(projPullsData$createdAt) <= end_date,]
    
    submitedPRs <- nrow(submitedPRs_df)
    
    mergedPRs_df <- projPullsData[projPullsData$merged &
                                  projPullsData$mergedAt >= start_date &
                                  projPullsData$mergedAt <= end_date,]
    
    mergedPRs <- nrow(mergedPRs_df)
    
    rejectedPRs_df <- projPullsData[!is.na(projPullsData$closedAt) &
                                    projPullsData$closedAt >= start_date &
                                    projPullsData$closedAt <= end_date &
                                    projPullsData$merged == FALSE,]
    
    rejectedPRs <- nrow(rejectedPRs_df)
    
    pullRequestContributors <- length(unique(submitedPRs_df$authorLogin))
    
    medianPRChurn <- median(submitedPRs_df$additions + submitedPRs_df$deletions)
    medianPRChangedFiles <- median(submitedPRs_df$changedFiles)
    medianPRCommits <- median(submitedPRs_df$commitsCount)
    medianPRMergeTimeInDays <- median(mergedPRs_df$mergeTimeInDays)
    medianPRRejectTimeInDays <- median(round(difftime(as.POSIXct(rejectedPRs_df$closedAt), as.POSIXct(rejectedPRs_df$createdAt), units = "days"), digits = 3))
    
    intervalOrder <- monthIntervals + 1
    
    interval <- data.frame(proj, loc_size, generalProjectCategory,
                           intervalOrder,
                           start_date, end_date, 
                           commitsCount, authors, weekendDaysInInterval,
                           commitRatePerDay, commitRatePerDayExcludingWeekendDays,
                           commitsPerAuthor,
                           commitActivity,
                           runsActivity,
                           runsInIntervalCount,
                           successfullyRunsInIntervalCount,
                           failedRunsInIntervalCount, 
                           otherRunsInIntervalCount,
                           medianRunDurationInMinutes,
                           fixedRunsInInterval,
                           medianTimeToFixRunsInHours,
                           runsHealth,
                           lastCoverageInInterval,
                           submitedPRs,
                           mergedPRs,
                           rejectedPRs,
                           pullRequestContributors,
                           medianPRChurn,
                           medianPRChangedFiles,
                           medianPRCommits,
                           medianPRMergeTimeInDays,
                           medianPRRejectTimeInDays
    )
    repos_month_date_intervals <- rbind(repos_month_date_intervals, interval)
    
    start_date <- end_date + days(1)
    monthIntervals  <- monthIntervals + 1
  }
  
  repos_data[repos_data$fullName == proj, 'monthlyIntervals'] <- monthIntervals
}

# -----------------------------
# Calculating the difference in percentage of the monthly changes in medianRunDurationInMinutes
# for the studied projects

repos_month_date_intervals <- repos_month_date_intervals %>%
  arrange(proj, intervalOrder) %>%
  group_by(proj) %>%
  mutate(Last_Non_Null_Value = zoo::na.locf(lag(medianRunDurationInMinutes), na.rm = FALSE, fromLast = FALSE)) %>%
  mutate(Percentage_Difference = ifelse(is.na(Last_Non_Null_Value), NA, (medianRunDurationInMinutes - Last_Non_Null_Value) / Last_Non_Null_Value * 100)) %>%
  select(-c(medianRunDurationInMinutes), everything(), medianRunDurationInMinutes, Last_Non_Null_Value, Percentage_Difference)


repos_month_date_intervals_filename <- "repos_month_date_intervals.csv"
write.csv(repos_month_date_intervals[, -which(names(repos_month_date_intervals) %in% c("Last_Non_Null_Value", "X", "X.1"))], file = repos_month_date_intervals_filename, row.names = FALSE, na = c('NULL'))

# Define the order of the boxes
loc_size_box_order <- c("small", "medium", "large")
box_order <- c("ML", "No ML")

# Reorder the factor levels
repos_month_date_intervals$loc_size <- reorder(repos_month_date_intervals$loc_size, as.numeric(factor(repos_month_date_intervals$loc_size, levels = loc_size_box_order)))
repos_month_date_intervals$generalProjectCategory <- reorder(repos_month_date_intervals$generalProjectCategory, as.numeric(factor(repos_month_date_intervals$generalProjectCategory, levels = box_order)))

###############################################################################
#
# ADDING THE MEDIAN VALUES FOR THE CI METRICS TO THE REPOSITORY DATASET
#
###############################################################################

repos_ci_metrics_median <-
  summarise(group_by(repos_month_date_intervals, proj), 
            medianCommitActivity = median(commitActivity, na.rm = TRUE),
            medianCommitRatePerDay = median(commitRatePerDay, na.rm = TRUE),
            medianLastCoverageInInterval = median(lastCoverageInInterval, na.rm = TRUE),
            medianTimeToFixRunsInHours = median(medianTimeToFixRunsInHours, na.rm = TRUE),
            medianRunDurationInMinutes = median(medianRunDurationInMinutes, na.rm = TRUE))

names(repos_ci_metrics_median)[1] <- "fullName"

## Adding the repos_ci_metrics columns to the repos_data dataset
repos_data <- left_join(repos_data,
                        repos_ci_metrics_median,
                        by = "fullName")


write.csv(repos_data, file = "repositories-info_with_ci_metrics.csv", row.names = FALSE, na = c('NULL'))

View(repos_data)
View(repos_month_date_intervals)
